/*
(c) 2001-2019 various contributors (see LICENSE)
(c) 2020 Pascal R. G. Abresch (MIT License)
*/

#include "AddContactWindow.h"

#include <Application.h>

#include "support/AppLocation.h"

#include "ui/ModalAlertFactory.h"
#include "ui/PictureView.h"

#include <interface/Button.h>

#include "jabber/Agent.h"
#include "jabber/AgentList.h"
#include "jabber/BlabberSettings.h"
#include "jabber/GenericFunctions.h"
#include "jabber/Messages.h"
#include "jabber/JRoster.h"

#include <cstdio>
#include <string.h>

#include "GridView.h"
#include "GroupLayout.h"
#include "GroupLayoutBuilder.h"
#include "StringView.h"

AddContactWindow *AddContactWindow::sInstance = NULL;

AddContactWindow *AddContactWindow::Instance() {
	if (sInstance == NULL) {
		BRect frame(GenericFunctions::CenteredFrame(440, 165)); //TODO: remove this?
		sInstance = new AddContactWindow(frame);
	}

	return sInstance;
}


AddContactWindow::~AddContactWindow() {
	sInstance = NULL;
}


AddContactWindow::AddContactWindow(BRect frame)
	: BWindow(frame, "Add Contact", B_TITLED_WINDOW, B_AUTO_UPDATE_SIZE_LIMITS | B_NOT_ZOOMABLE) {

	fNickName = new BTextControl("realname", "Nickname:", "", NULL);
	fNickName->SetModificationMessage(new BMessage(JAB_NICK_CHANGED));

	fXMPPId = new BTextControl("handle", "XMPP ID:", "", NULL);
	fXMPPId->SetModificationMessage(new BMessage(JAB_ID_CHANGED));

	BStringView* inputHint = new BStringView("enter", "Example: username@server.com", B_WILL_DRAW);

	BButton* cancelButton = new BButton("Cancel", new BMessage(B_QUIT_REQUESTED));
	cancelButton->SetTarget(this);

	BButton* okButton = new BButton("ok", new BMessage(JAB_OK));
	okButton->MakeDefault(true);
	okButton->SetTarget(this);
	okButton->SetLabel("Add Contact");

	SetLayout(new BGroupLayout(B_VERTICAL));
	AddChild(BGroupLayoutBuilder(B_VERTICAL, 4)
		.Add(fNickName)
		.Add(fXMPPId)
		.Add(inputHint)
			.Add(BGroupLayoutBuilder(B_HORIZONTAL, 2)
			.Add(okButton)
			.Add(cancelButton))
		);
	fNickName->MakeFocus(true);
}


void AddContactWindow::MessageReceived(BMessage *msg) {
	switch (msg->what) {
		case JAB_OK: {
			AddNewUser();
			break;
		}

		case JAB_NICK_CHANGED: {
			if (strcmp(fNickName->Text(), "")) {
				fNickName->MarkAsInvalid(false);
			}
			break;
		}

		case JAB_ID_CHANGED: {
			if (fXMPPIdGloox.setJID(fXMPPId->Text())) {
				fXMPPId->MarkAsInvalid(false);
			}
			break;
		}

	}
}


bool AddContactWindow::QuitRequested() {
	sInstance = NULL;
	return true;
}


void AddContactWindow::AddNewUser() {
	//char buffer[4096];

	if (!strcmp(fNickName->Text(), "")) {
		fNickName->MarkAsInvalid(true);
		fNickName->MakeFocus(true);
		return;
	}

	if (!fXMPPIdGloox.setJID(fXMPPId->Text())) {
		fXMPPId->MarkAsInvalid(true);
		fXMPPId->MakeFocus(true);
		return;
	}

	// make sure it's not a duplicate of one already existing (unless itself)
	/*JRoster::Instance()->Lock();
	if (JRoster::Instance()->FindUser(JRoster::COMPLETE_HANDLE, fXMPPId->Text())) {
		sprintf(buffer, "%s already exists in your buddy list.  Please choose another so you won't get confused.", fXMPPId->Text());
		ModalAlertFactory::Alert(buffer, "Good Idea!");
		fXMPPId->MakeFocus(true);

		JRoster::Instance()->Unlock();
		return;
	}
	JRoster::Instance()->Unlock();
	*/

	// add this user to the roster
	JRoster::Instance()->Lock();
	JRoster::Instance()->AddNewUser(fXMPPIdGloox, fNickName->Text());
	JRoster::Instance()->Unlock();

	// alert all RosterViews
	JRoster::Instance()->RefreshRoster();

	// close window explicitly
	PostMessage(B_QUIT_REQUESTED);
}
